	// make RMEQCalc into EQSpec1
	// then EQ1 is subclass which manages a synth object on server


EQSpec1 {
		// takes type, freq, rq and k and turns them into SOS coefficients
		// form borrowed from Casey Basichis
		// sc form of Regalia-Mitra equations borrowed from Joseph Anderson
		
	var	<type = \eq,	// can be \eq, \hishelf, \loshelf
		<freq = 440,
		<k = 1.0,		// scaling factor: < 1.0 for cut, > 1.0 for boost
		<rq = 1.0;		// reciprocal of q
		
	var	<>sampleRate = 44100;	// a big kludge -- how do I ask a server its samplerate?
	
		// coefficients
	var	<a0, <a1;
	
	*newSpec { arg type = \eq, freq = 440, k = 1.0, rq = 1.0, sr = 44100;
		^super.newCopyArgs(type, freq, k, rq, sr).calc;
	}
	
	calc {
		var wc, y, tanw;
		
		(type == \eq).if({
			wc = 2pi * freq * sampleRate.reciprocal;
			y = wc.cos.neg;
			tanw = (0.5 * wc * rq).tan;
	
			a0 = (1 - tanw) / (1 + tanw);
			a1 = y * (1 + a0);

			^this
//			^[freq, k, rq, a0, a1]
		});
		
		(type == \hishelf).if({
			k = k.abs;	// positive values for gain at nyquist
			wc = pi * freq * sampleRate.reciprocal;
			a0 = (1 - wc)/(1 + wc);
			
			^this
//			^[freq, k, b1]
		});
		
		(type == \loshelf).if({
			k = k.abs.neg;	// neg values for gain at DC (0Hz)
			wc = pi * freq * sampleRate.reciprocal;
			a0 = (1 - wc)/(1 + wc);
			
			^this
//			^[freq, k, b1]
		});
		("EQ type \\" ++ type ++ " is invalid.").error;
		this.verboseHalt;
	}
	
	synthArgs {
		(type == \eq).if({
			^[\a0, a0, \a1, a1, \k, k]
		}, {
			^[\b1, a0, \k, k]
		});
	}
	
	type_ { arg t;
		type = t;
		this.calc;
	}
	
	freq_ { arg f;
		freq = f;
		this.calc;
	}
	
	k_ { arg newk;
		k = newk;
		this.calc;
	}
	
	rq_ { arg r;
		rq = r;
		this.calc;
	}
	
}

EQBand : EQSpec1 {
	// contains specs for 1 EQ filter
	// 1 band of stereo eq takes only 1.1% of cpu on a 700MHz iBook
	// this will go down when I make a ugen plugin for eq
	
	classvar	<translateMethods;	// to change add method to move method
	
	var	mul = 1,
		numChannels = 1,
		target, bus,
		synth;
	
	*initClass {
			// if the band is already playing, this.play(target) moves the synth
			// rather than making a new one. This dictionary converts add messages
			// into move messages.
		translateMethods = IdentityDictionary[
			\addToHead -> \moveToHead,
			\addToTail -> \moveToTail,
			\addAfter -> \moveAfter,
			\addBefore -> \moveBefore,
			\moveToHead -> \moveToHead,	// b/c user might specify a move method
			\moveToTail -> \moveToTail,	// this makes sure we don't choke
			\moveAfter -> \moveAfter,
			\moveBefore -> \moveBefore
		];
	}
	
	*new { arg type = \eq, freq = 440, k = 1, rq = 1, numChannels = 1, sr = 44100;
		^super.new.initBand(type, freq, k, rq, numChannels, sr);
	}
	
	initBand { arg t, f, gain, recq, numCh, sampRt;
		type = t;
		freq = f;
		k = gain;
		rq = recq;
		numChannels = numCh;
		sampleRate = sampRt;
		this.calc;
		^this
	}
		
	*newFromArray { arg a, numChan = 1, sr = 44100;	// array comes from EQSpecs parsing algorithm
		^this.new(a.at(0) ? \eq, a.at(1) ? 440, a.at(2) ? 1, a.at(3) ? 1,
			numChan, sr
		)
	}
	
		// place the eq band on the server
	play { arg target, bus, addAction = \addToTail, amp = 1;
		var bundle;
			// fix target
		target.isKindOf(Group).not.if({	// target is Group, leave it alone, otherwise...
			target.isKindOf(Synth).if({
				addAction = \addAfter;	// if a synth, place imm. after target
			});
			target.isKindOf(MixerChannel).if({	// if mixerchannel,
				bus = target.inbus;	// must use mc bus
				target = target.effectgroup;	// place where effects should go
			});
			target = target.asTarget;
		});
		
		bus.isKindOf(Integer).if({	// send an index, make a bus from it
			bus = Bus.new(\audio, bus, numChannels, target.server);
		});
		
		bus.isNil.if({	// give nothing for bus, use hardware on target's server
			bus = Bus.new(\audio, 0, numChannels, target.server);
		});
		
		mul = amp;

		synth.isNil.if({		// make new synth, else move it using action and target
			bundle = List.new;
			synth = Synth.newToBundle(bundle, "EQ/"++type.asString++numChannels,
				this.calc.synthArgs ++ [\outbus, bus.index, \mul, mul], target, addAction);
			target.server.listSendBundle(nil, bundle);
		}, {
			synth.perform(translateMethods.at(addAction), target);
		});
	}
	
		// stop the eq, free server resources
	free {
		synth.notNil.if({ synth.free; });
		synth = nil;
	}
	
	mul_ { arg m;
		mul = m;
		synth.set([\mul, mul]);
	}

	type_ { arg t;
		var bundle;
		type = t;
		
			// if type changes, synth node must be replaced
		bundle = List.new;
		synth = Synth.newToBundle(bundle, "EQ/"++type.asString++numChannels,
			this.calc.synthArgs ++ [\outbus, bus.index], synth, \addReplace);
		target.server.listSendBundle(nil, bundle);
	}
	
	freq_ { arg f;
		freq = f;
		this.calc;
		synth.notNil.if({		// if the synth is on, give it the new values
			synth.server.sendBundle(nil, [15, synth.nodeID]++this.synthArgs.postln);
		});
	}
	
	k_ { arg newk;
		k = newk;
		this.calc;
		synth.notNil.if({
			synth.server.sendBundle(nil, [15, synth.nodeID]++this.synthArgs.postln);
		});
	}
	
	rq_ { arg r;
		rq = r;
		this.calc;
		synth.notNil.if({
			synth.server.sendBundle(nil, [15, synth.nodeID]++this.synthArgs.postln);
		});
	}
	
}